@@ -5,7 +5,7 @@
 ;; Author: Adam Porter <adam@alphapapa.net>
 ;; Maintainer: Adam Porter <adam@alphapapa.net>
 ;; URL: https://github.com/alphapapa/ement.el
-;; Version: 0.5.2
+;; Version: 0.6
 ;; Package-Requires: ((emacs "27.1") (map "2.1") (plz "0.2") (taxy "0.10") (taxy-magit-section "0.12.1") (svg-lib "0.2.5") (transient "0.3.7"))
 ;; Keywords: comm
 
@@ -236,7 +236,9 @@ the port, e.g.
                () (pcase-let* (((cl-struct ement-session user device-id initial-device-display-name) session)
                                ((cl-struct ement-user id) user)
                                (data (ement-alist "type" "m.login.password"
-                                                  "user" id
+                                                  "identifier"
+                                                  (ement-alist "type" "m.id.user"
+                                                               "user" id)
                                                   "password" password
                                                   "device_id" device-id
                                                   "initial_device_display_name" initial-device-display-name)))
@@ -450,21 +452,29 @@ a filter ID).  When unspecified, the value of
                           :then (apply-partially #'ement--sync-callback session)
                           :else (lambda (plz-error)
                                   (setf (map-elt ement-syncs session) nil)
-                                  (pcase (plz-error-curl-error plz-error)
-                                    (`(,(or 28 429 502) . ,_)
-                                     ;; Timeout (28), "Too Many Requests" (429), or "Bad Gateway" (502): sync again if enabled.
-                                     (if (not ement-auto-sync)
-                                         (run-hook-with-args 'ement-interrupted-sync-hook session)
-                                       (let ((reason (pcase-exhaustive (car (plz-error-curl-error plz-error))
-                                                       (28 "timed out")
-                                                       ((or 429 502) "failed"))))
-                                         (message "Ement: Sync %s (%s).  Syncing again..."
-                                                  reason (ement-user-id (ement-session-user session))))
-                                       ;; Set QUIET to allow the just-printed message to remain visible.
-                                       (ement--sync session :timeout timeout :quiet t)))
-                                    (`(,code . ,message)
-                                     (signal 'ement-api-error (list (format "Ement: Network error: %s: %s" code message) plz-error)))
-                                    (_ (signal 'ement-api-error (list "Ement: Unrecognized network error" plz-error)))))
+                                  ;; TODO: plz probably needs nicer error handling.
+                                  ;; Ideally we would use `condition-case', but since the
+                                  ;; error is signaled in `plz--sentinel'...
+                                  (pcase-let (((cl-struct plz-error curl-error response) plz-error)
+                                              (reason))
+                                    (cond ((when response
+                                             (pcase (plz-response-status response)
+                                               ((or 429 502) (setf reason "failed")))))
+                                          ((pcase curl-error
+                                             (28 (setf reason "timed out")))))
+                                    (if reason
+                                        (if (not ement-auto-sync)
+                                            (run-hook-with-args 'ement-interrupted-sync-hook session)
+                                          (message "Ement: Sync %s (%s).  Syncing again..."
+                                                   reason (ement-user-id (ement-session-user session)))
+                                          ;; Set QUIET to allow the just-printed message to remain visible.
+                                          (ement--sync session :timeout timeout :quiet t))
+                                      ;; Unrecognized errors:
+                                      (pcase curl-error
+                                        (`(,code . ,message)
+                                         (signal 'ement-api-error (list (format "Ement: Network error: %s: %s" code message)
+                                                                        plz-error)))
+                                        (_ (signal 'ement-api-error (list "Ement: Unrecognized network error" plz-error)))))))
                           :json-read-fn (lambda ()
                                           "Print a message, then call `json-read'."
                                           (when (ement--sync-messages-p session)
